
local addonName,adapt = ...

local opt = CreateFrame("Frame",nil,InterfaceOptionsFramePanelContainer)

local template = {
	[1] = {"label",GetAddOnMetadata(addonName,"Title"),"GameFontNormalLarge","TOPLEFT",0,"TOPLEFT",16,-16},
	[2] = {"label","version "..GetAddOnMetadata(addonName,"Version"),"GameFontNormalSmall","BOTTOMLEFT",1,"BOTTOMRIGHT",4,0},
	[3] = {"label","This addon changes static portraits to animated models.","GameFontHighlightSmall","TOPLEFT",1,"BOTTOMLEFT",0,-8},
	[4] = {"label","These options affect how portraits are displayed:","GameFontNormal","TOPLEFT",3,"BOTTOMLEFT",0,-32},
	[5] = {"check","Square","Square portraits","TOPLEFT",4,"BOTTOMLEFT",8,-16},
	[6] = {"check","FullModel","Full model","TOPLEFT",5,"BOTTOMLEFT",0,2},
	[7] = {"label","Background color","GameFontHighlight","TOPLEFT",6,"BOTTOMLEFT",28,-15},
	[8] = {"label","or","GameFontHighlight","TOPLEFT",7,"BOTTOMLEFT",12,-4},
	[9] = {"check","ColorByClass","Color background by class","TOPLEFT",6,"BOTTOMLEFT",0,-40},
	[10]= {"label","These options allow you to turn off the animation of some portraits:","GameFontNormal","TOPLEFT",9,"BOTTOMLEFT",-8,-32},
	[11]= {"label","Some addons do not name their potraits. Put an \124TInterface\\RAIDFRAME\\ReadyCheck-NotReady:16\124t here to not animate them:","GameFontNormalSmall","TOPLEFT",10,"BOTTOMLEFT",8,-16},
	[12]= {"check","BlacklistAnonymous","Anonymous portraits","TOPLEFT",11,"BOTTOMLEFT",14,-4},
	[13]= {"label","To prevent a named portrait from animating, put an \124TInterface\\RAIDFRAME\\ReadyCheck-NotReady:16\124t beside its name:","GameFontNormalSmall","TOPLEFT",11,"BOTTOMLEFT",-0,-36},
	[14]= {"color","BackColor",16,16,"RIGHT",7,"LEFT",-7,0},
	[15]= {"image","Interface\\AddOns\\Adapt\\CircleBorder","OVERLAY",112,112,"TOPRIGHT",0,"TOPRIGHT",-96,-104},
}

-- addon should call this in its PLAYER_LOGIN
function adapt:InitializeOptions(savedvar,defaultvar)
	opt:Hide()
	opt.name = addonName
	opt.widgets = {} -- indexed by pointers to checkboxes, editboxes, etc
	opt.backup = {} -- in case we cancel changes, restore to backup
	opt.savedvar = savedvar -- table of savedvariables used by main addon
	opt.defaultvar = defaultvar -- table of default savedvariables
	opt:BuildUI()
	InterfaceOptions_AddCategory(opt)

end

function adapt:ShowOptions()
	opt:refresh()
	InterfaceOptionsFrame_OpenToCategory(addonName) 
end

function opt:refresh()
	wipe(self.backup)
	-- backup existing settings and update widgets to current value
	for i=1,#template do
		local widgetType = template[i][1]
		if widgetType=="check" then
			self.backup[i] = self.savedvar[template[i][2]]
			self.widgets[i]:SetChecked(self.backup[i])
		elseif widgetType=="color" then
			local saved = self.savedvar[template[i][2]]
			self.backup[i] = {r=saved.r, g=saved.g, b=saved.b, a=saved.a}
			self.widgets[i].fg:SetVertexColor(saved.r,saved.g,saved.b,saved.a)
		end
	end
	-- adapt-specific stuff (options portrait, blacklist)
	SetPortraitTexture(AdaptOptionsPortrait,"player")
	opt:ShapeOptionsPortraitBorder()
	opt.portraitList = opt.portraitList or {}
	wipe(opt.portraitList)
	opt.backup.blacklist = {}
	-- populate observed portraits
	for texture,portrait in pairs(adapt.portraits) do
		local name = texture:GetName()
		if name and portrait.cx>28 and texture~=AdaptOptionsPortrait then
			table.insert(opt.portraitList,name)
			opt.backup.blacklist[name] = AdaptSettings.Blacklist[name]
		end
	end
	-- add any portraits already on the blacklist not observed yet
	for name in pairs(AdaptSettings.Blacklist) do
		if not tContains(opt.portraitList,name) then
			table.insert(opt.portraitList,name)
			opt.backup.blacklist[name] = 1
		end
	end
	table.sort(opt.portraitList)
	opt.list.scrollFrame.ScrollBar:SetValue(0)
	opt:BlacklistScrollFrameUpdate()
	self.changed = nil
end

function opt:okay()
	for i=1,#template do
		if template[i][1]=="check" then
			self.savedvar[template[i][2]] = self.widgets[i]:GetChecked()
		end
	end
	if opt.changed then
		adapt.RefreshAll()
	end
end

function opt:cancel()
	for i=1,#template do
		if template[i][1]=="check" then
			self.savedvar[template[i][2]] = self.backup[i]
		end
	end
	wipe(AdaptSettings.Blacklist)
	for textureName in pairs(opt.backup.blacklist) do
		AdaptSettings.Blacklist[textureName] = 1
	end
	adapt.UpdateBlackcache()
	if self.changed then
		adapt.RefreshAll()
	end
end

function opt:default()
	if self.defaultvar then
		wipe(self.savedvar)
		for k,v in pairs(self.defaultvar) do
			if type(v)=="table" then
				self.savedvar[k] = {}
				for x,y in pairs(v) do
					self.savedvar[k][x] = y
				end
			else
				self.savedvar[k] = v
			end
		end
	end
	adapt.RefreshAll()
end

--[[ UI ]]

function opt:CreateText(text,font,anchorPoint,relativeTo,relativePoint,xoff,yoff)
	local fontstring = opt:CreateFontString(nil,"BACKGROUND",font)
	relativeTo = relativeTo==0 and opt or opt.widgets[relativeTo]
	fontstring:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	fontstring:SetJustifyH("LEFT")
	fontstring:SetText(text)
	return fontstring
end

function opt:CreateCheck(variable,text,anchorPoint,relativeTo,relativePoint,xoff,yoff)
	local check = CreateFrame("CheckButton",nil,opt,"UICheckButtonTemplate")
	relativeTo = relativeTo==0 and opt or opt.widgets[relativeTo]
	check.variable = variable
	check:SetSize(26,26)
	check:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	check.text:SetFontObject("GameFontHighlight")
	check.text:SetText(" "..text)
	check:SetScript("OnClick",opt[variable.."OnClick"])
	return check
end

function opt:SetCheckedByName(name,checked)
	for i=1,#template do
		if template[i][2]==name then
			opt.widgets[i]:SetChecked(checked)
		end
	end
end

function opt:CreateColorSwatch(ctable,cx,cy,anchorPoint,relativeTo,relativePoint,xoff,yoff)
	local swatch = CreateFrame("Button",nil,opt)
	relativeTo = relativeTo==0 and opt or opt.widgets[relativeTo]
	swatch:SetSize(cx,cy)
	swatch:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	swatch.bg = swatch:CreateTexture(nil,"BACKGROUND")
	swatch.bg:SetAllPoints(true)
	swatch.bg:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	swatch.bg:SetVertexColor(.65,.65,.65)
	swatch.mid = swatch:CreateTexture(nil,"BORDER")
	swatch.mid:SetAllPoints(true)
	swatch.mid:SetTexture("Interface\\FriendsFrame\\Battlenet-WoWicon")
	swatch.fg = swatch:CreateTexture(nil,"ARTWORK")
	swatch.fg:SetPoint("TOPLEFT",swatch,"TOPLEFT",-1,1)
	swatch.fg:SetPoint("BOTTOMRIGHT",swatch,"BOTTOMRIGHT",1,-1)
	swatch.fg:SetTexture("Interface\\ChatFrame\\ChatFrameColorSwatch")
	swatch:SetScript("OnEnter",function(self) self.bg:SetVertexColor(1,.82,0) end)
	swatch:SetScript("OnLeave",function(self) self.bg:SetVertexColor(.65,.65,.65) end)
	swatch:SetScript("OnClick",opt[ctable.."OnClick"])
	return swatch
end

function opt:CreateImage(texture,layer,cx,cy,anchorPoint,relativeTo,relativePoint,xoff,yoff)
	local image = opt:CreateTexture(nil,layer)
	relativeTo = relativeTo==0 and opt or opt.widgets[relativeTo]
	image:SetSize(cx,cy)
	image:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	image:SetTexture(texture)
	return image
end

function opt:BuildUI()
	for i=1,#template do
		local widgetType = template[i][1]
		if widgetType=="label" then
			self.widgets[i] = self:CreateText(select(2,unpack(template[i])))
		elseif widgetType=="check" then
			self.widgets[i] = self:CreateCheck(select(2,unpack(template[i])))
		elseif widgetType=="color" then
			self.widgets[i] = self:CreateColorSwatch(select(2,unpack(template[i])))
		elseif widgetType=="image" then
			self.widgets[i] = self:CreateImage(select(2,unpack(template[i])))
		end
		self.widgets[i].id = i
	end

	local portrait = opt:CreateTexture("AdaptOptionsPortrait","ARTWORK")
	portrait:SetPoint("TOPLEFT",self.widgets[15],"TOPLEFT",5,-5)
	portrait:SetPoint("BOTTOMRIGHT",self.widgets[15],"BOTTOMRIGHT",-5,5)

	self.widgets[12]:SetCheckedTexture("Interface\\RAIDFRAME\\ReadyCheck-NotReady")

	opt:CreateBlacklist()
end

--[[ Adapt-specific options stuff ]]

function opt:SquareOnClick()
	opt.changed = 1
	AdaptSettings.Square = self:GetChecked()
	opt:ShapeOptionsPortraitBorder()
	adapt.ShapeAllModels()
end

function opt:ShapeOptionsPortraitBorder()
	opt.widgets[15]:SetTexture(AdaptSettings.Square and "Interface\\AddOns\\Adapt\\SquareBorder" or "Interface\\AddOns\\Adapt\\CircleBorder")
end

function opt:FullModelOnClick()
	opt.changed = 1
	AdaptSettings.FullModel = self:GetChecked()
	adapt.SetAllCameras()
end

function opt:ColorByClassOnClick()
	opt.changed = 1
	AdaptSettings.ColorByClass = self:GetChecked()
	adapt.ColorAllBackLayers()
end

function opt:BlacklistAnonymousOnClick()
	opt.changed = 1
	AdaptSettings.BlacklistAnonymous = self:GetChecked()
end

function opt:BackColorChanged(...)
	local back = AdaptSettings.BackColor
	back.r, back.g, back.b = ColorPickerFrame:GetColorRGB()
	back.a = 1-OpacitySliderFrame:GetValue()
	adapt.ColorAllBackLayers()
	opt.widgets[opt.modalwidget].fg:SetVertexColor(back.r,back.g,back.b,back.a)
end

function opt:BackColorCancelled(...)
	local back = AdaptSettings.BackColor
	back.r, back.g, back.b, back.a = unpack(ColorPickerFrame.previousValues)
	AdaptSettings.ColorByClass = opt.oldColorByClass
	opt:SetCheckedByName("ColorByClass",AdaptSettings.ColorByClass)
	adapt.ColorAllBackLayers()
	opt.widgets[opt.modalwidget].fg:SetVertexColor(back.r,back.g,back.b,back.a)
end

function opt:BackColorOnClick()
	opt.changed = 1
	opt.oldColorByClass = AdaptSettings.ColorByClass
	AdaptSettings.ColorByClass = nil
	opt:SetCheckedByName("ColorByClass",nil)
	local saved = AdaptSettings.BackColor
	ColorPickerFrame.func = opt.BackColorChanged
	ColorPickerFrame.opacityFunc = opt.BackColorChanged
	ColorPickerFrame.cancelFunc = opt.BackColorCancelled
	opt.modalwidget = self.id
	ColorPickerFrame.hasOpacity = true
	ColorPickerFrame.opacity = 1-saved.a
	ColorPickerFrame.previousValues = {saved.r,saved.g,saved.b,saved.a}
	ColorPickerFrame:SetColorRGB(saved.r,saved.g,saved.b)
	ColorPickerFrame:Hide()
	ColorPickerFrame:Show()
end

function opt:CreateBlacklist()
	opt.list = CreateFrame("Frame",nil,opt)
	opt.list:SetSize(300,176)
	opt.list:SetPoint("TOPLEFT",opt.widgets[13],"BOTTOMLEFT",8,-8)
	opt.list:SetBackdrop({bgFile="Interface\\DialogFrame\\UI-DialogBox-Background", insets={left=4,right=4,top=4,bottom=4}, tileSize=16, tile=true, edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", edgeSize=16})
	opt.list:SetBackdropBorderColor(.5,.5,.5,1)
	for i=1,8 do
		opt.list[i] = CreateFrame("CheckButton",nil,opt.list,"UICheckButtonTemplate")
		opt.list[i]:SetSize(24,24)
		opt.list[i]:SetPoint("TOPLEFT",8,(i-1)*-20-6)
		opt.list[i].text:SetFontObject("GameFontHighlight")
		opt.list[i].text:SetPoint("LEFT",opt.list[i],"RIGHT",2,1)
		opt.list[i]:SetHitRectInsets(0,-240,0,0)
		opt.list[i]:SetCheckedTexture("Interface\\RAIDFRAME\\ReadyCheck-NotReady")
		opt.list[i]:SetScript("OnClick",opt.BlacklistCheckOnClick)
	end
	opt.list.scrollFrame = CreateFrame("ScrollFrame","AdaptBlacklistScrollFrame",opt.list,"FauxScrollFrameTemplate")
	opt.list.scrollFrame:SetPoint("TOPLEFT",0,-8)
	opt.list.scrollFrame:SetPoint("BOTTOMRIGHT",-28,6)

	opt.list.scrollFrame:SetScript("OnShow",opt.BlacklistScrollFrameUpdate)
	opt.list.scrollFrame:SetScript("OnVerticalScroll",function(self,offset)
		FauxScrollFrame_OnVerticalScroll(self,offset,20,opt.BlacklistScrollFrameUpdate)
	end)
end

function opt:BlacklistCheckOnClick()
	opt.changed = 1
	local portraitName = self.text:GetText()
	AdaptSettings.Blacklist[portraitName] = self:GetChecked()
end

function opt:BlacklistScrollFrameUpdate()
	local offset = FauxScrollFrame_GetOffset(opt.list.scrollFrame)
	FauxScrollFrame_Update(opt.list.scrollFrame, #opt.portraitList, 8, 20)
	for i=1,8 do
		local idx = offset+i
		if idx<=#opt.portraitList then
			local portraitName = opt.portraitList[idx]
			opt.list[i].text:SetText(portraitName)
			opt.list[i]:SetChecked(AdaptSettings.Blacklist[portraitName])
			opt.list[i]:Show()
		else
			opt.list[i]:Hide()
		end
	end
end
